#
# The Poll Tag
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from DocumentTemplate.DT_Util import *

from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import HTMLFile, HTML, MessageDialog
from Acquisition import Implicit
from OFS.Folder import Folder
from OFS.Image import getImageInfo
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse
from urllib import quote

def manage_addPoll(self, em='', na='', qu='', votes=[], REQUEST={}):
	""" Add a New Poll Question """

	error=''

	for voteid in self.objectValues('Poll'):
		if voteid.question == qu:
			error=error+("There's already a question about this "+
						 "(Did you reload this page? "+
						 "You shouldn't do that, you know...)<br>"
						 )
			break

	if not qu:
		    error=error+("Please, you must write a QUESTION! "+
				 "Please go back and fix this!<p>")
	elif(len(qu) < 6 or len(qu) > 90):
		    error=error+(
			"The question you submitted (\""+qu+"\") "+
			"is not correct. "+
			"Either its too long and complex, or too short and "+
			"simple. Please keep within six to 90 characters!<p>"
			)
	if(em==""):
		error=error+(
			"You must give us your email address! please go "+
			"back and fix this!<p>"
			)
	elif ((len(em)<3) or (string.find(em, "@")==-1)):
		error=error+("The email adress you submitted (\""+
					 em+"\") is not correct. "+
					 "Please fix this!<p>"
					 )
	if (na==""):
		error=error+(
			"You must give us your name! please go back and "+
			"fix this!<p>"
			)
	elif ((len(na)<3) or (len(na)>30)):
		error=error+(
			"The name you gave us (\""+na+"\") is either too"+
			"long or short. Please fix this!<p>"
			)

	if (len(votes)==0):
		error = error+"You must enter some vote alternatives!"
	elif (len(votes)<2):
		error = error+"You must enter atleast two vote alternatives!"
	for vote in votes:
		if (len(vote) > 60):
			error = error+(
			    "The vote alternative \""+vote+"\" has too "+
			    "many characters in it. It's simply too long."
			    )

	if(error):
		return MessageDialog(
			title="Problems with new Poll",
			message=("<b>We experienced some problems with your new vote "+
					 "topic suggestion:</b><p>"+error+"<p>"+
					 "Please go back and fix these problem(s)!"),
			action="manage_main")

	id=str(time.time())
	obj=Poll(id=id, title=qu, question=qu,
			 email=em, name=na, votes=votes)
	self._setObject(id, obj)
	obj=getattr(self, id)
	obj.setStatus(self.SUGGESTED)
	
	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addPollForm=HTMLFile('manage_addPollForm', globals())
user_addPollForm=HTMLFile('user_addPollForm', globals())

class Poll(Folder):
	""" An Actual Poll """

	meta_type='Poll'

	do_neat_table=HTMLFile('do_neat_table',globals())
	qVotes={}
	suggested=0.0

	our_index_html=HTMLFile('poll_index_html', globals())
	our_results=HTMLFile('currentResults', globals())
	
	def __init__(self, id, title, question, email, name, votes):

		self.id=id
		self.title=title
		self.question=self.fixx(question)
		self.email=email
		self.name=self.fixx(name)
		self.ok=0
		self.totalVotes=0
		self.suggested=time.time()
		self.voted_today={}
		self.no_options=0
		for j in range(len(votes)):
			if votes[j]:
				self.qVotes[j]={'vote':0,
								'txt': self.fixx(votes[j])}
				self.no_options=self.no_options+1

		self.qVotes=self.qVotes

	# This is a little hack to allow us to acquire the real variables from
	# This class.
	def index_html(self, REQUEST):
		""" The Index File """
		return self.our_index_html(self, REQUEST)

	def setEndTime(self, date):
		self.endTime=date
		self._p_changed=1

	def setStartTime(self, date):
		self.startTime=date
		self._p_changed=1
	
	def setStatus(self, status):
		self.status=status
		self._p_changed=1

	def fixx(self,what):
		return(string.replace(string.replace(string.replace(what, "#", " "), "@", " "), "!", " "))

	thanks=HTMLFile('thanks',globals())
	sorry=HTMLFile('sorry', globals())

	def currentResults(self, REQUEST):
		return self.our_results(self, REQUEST)

	def vote(self, vote_id, REQUEST):
		""" Vote for Something """
		if int(vote_id) <= len(self.qVotes):
			ivote_id=int(vote_id)
			if ( self.allowmorevotes or
				 not self.voted_today.has_key(REQUEST['REMOTE_ADDR'])):
				self.qVotes[ivote_id]['vote']=self.qVotes[ivote_id]['vote']+1
				self.totalVotes=self.totalVotes+1
				self.voted_today[REQUEST['REMOTE_ADDR']]=(
					time.time())

				self.qVotes=self.qVotes
				self._p_changed=1
				
#				return self.thanks(self, REQUEST)

			else:
				pass
#				return self.sorry(self, REQUEST)

		# Probably need some type of 'error' page too
		return self.sorry(self,REQUEST)
		
